@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bulletin_board_ui.c,v 1.83.2.2 2003/12/29 09:58:14 fuyu Exp $
+ * $Id: bulletin_board_ui.c,v 1.86 2003/12/29 18:46:34 fuyu Exp $
  */
 
 #include "config.h"
@@ -48,6 +48,8 @@
 
 #include "worker.h"
 
+#include "ts_engine.h"
+
 #include <glib.h>
 #include <gtk/gtk.h>
 
@@ -55,10 +57,14 @@
 
 #include <pthread.h>
 
+#include <fcntl.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include <unistd.h>
+
 #include <zlib.h>
 
 
@@ -193,6 +199,17 @@ static GQuark application_id;
 guint threadlist_netstat_id;		/* ¼privateġġ*/
 static guint writing_netstat_id;
 
+/*
+ * ѡgdk_threads_enter()ʴĶǻȤ
+ */
+static TSEngine *engine = NULL;
+static TSCellHandle *list_entry_decorate_func;
+
+static const char *default_init_scm =
+  "(define (list-entry-decorate-default new-responses filter-enabled filter-ok new-thread favorite hidden) (cond (hidden (cons list-entry-fg-hidden list-entry-bg-hidden)) (new-thread (cons list-entry-fg-strong list-entry-bg-strong)) (filter-ok (cons (if new-responses list-entry-fg-emph list-entry-fg-normal) (if filter-enabled list-entry-bg-normal list-entry-bg-emph))) nil))"
+  "(define (list-entry-decorate-default-old new-responses filter-enabled filter-ok new-thread favorite hidden) (cons (if hidden list-entry-fg-hidden (if (and new-responses favorite) list-entry-fg-emph list-entry-fg-normal)) (if new-thread list-entry-bg-strong (if (and new-responses filter-ok) list-entry-bg-emph list-entry-bg-normal))))"
+  "(define list-entry-decorate list-entry-decorate-default)";
+
 
 void
 prepare_board_ui_initialization(OchushaApplication *application)
@@ -219,6 +236,89 @@ prepare_board_ui_initialization(OchushaApplication *application)
 
   board_info_id = g_quark_from_static_string("BulletinBoardUI::BoardInfo");
   application_id = g_quark_from_static_string("BulletinBoardUI::Application");
+
+
+  /* TSEngineν */
+  engine = ts_engine_new(TRUE);
+  if (engine->is_busy)
+    {
+      fprintf(stderr, "Couldn't initialize scheme engine...disabled.\n");
+      g_object_unref(G_OBJECT(engine));
+      engine = NULL;
+    }
+  else
+    {
+      FILE *scm_file;
+      int fd;
+
+#if DEBUG_TS_ENGINE
+      ts_engine_set_output_file(engine, stderr);
+#endif
+
+      /*  */
+      ts_engine_define_global_long(engine, "list-entry-fg-normal",
+				   LIST_ENTRY_FOREGROUND_NORMAL);
+      ts_engine_define_global_long(engine, "list-entry-fg-emph",
+				   LIST_ENTRY_FOREGROUND_EMPH);
+      ts_engine_define_global_long(engine, "list-entry-fg-strong",
+				   LIST_ENTRY_FOREGROUND_STRONG);
+      ts_engine_define_global_long(engine, "list-entry-fg-hidden",
+				   LIST_ENTRY_FOREGROUND_HIDDEN);
+
+      ts_engine_define_global_long(engine, "list-entry-bg-normal",
+				   LIST_ENTRY_BACKGROUND_NORMAL);
+      ts_engine_define_global_long(engine, "list-entry-bg-emph",
+				   LIST_ENTRY_BACKGROUND_EMPH);
+      ts_engine_define_global_long(engine, "list-entry-bg-strong",
+				   LIST_ENTRY_BACKGROUND_STRONG);
+      ts_engine_define_global_long(engine, "list-entry-bg-hidden",
+				   LIST_ENTRY_BACKGROUND_HIDDEN);
+
+      list_entry_decorate_func
+	= ts_engine_mk_cell_symbol(engine, "list-entry-decorate");
+
+
+      /* ѥǥեȽեɤ߹ */
+      scm_file = fopen(PKGDATADIR "/" OCHUSHA_INIT_SCM, "r");
+      if (scm_file != NULL)
+	{
+	  ts_engine_load_file(engine, scm_file);
+	  fclose(scm_file);
+	}
+      else
+	{
+	  ts_engine_load_string(engine, (char *)default_init_scm);
+#if DEBUG_TS_ENGINE
+	  fprintf(stderr, "Couldn't locate default %s\n", OCHUSHA_INIT_SCM);
+#endif
+	}
+
+      /* ѥ桼եɤ߹ */
+      fd = ochusha_config_open_file(&application->config, OCHUSHA_INIT_SCM,
+				    NULL, O_RDONLY);
+      if (fd > 0)
+	{
+	  scm_file = fdopen(fd, "r");
+	  if (scm_file != NULL)
+	    {
+	      ts_engine_load_file(engine, scm_file);
+	      fclose(scm_file);
+	    }
+	  else
+	    {
+	      close(fd);
+#if DEBUG_TS_ENGINE
+	      fprintf(stderr, "Couldn't open user's %s\n", OCHUSHA_INIT_SCM);
+#endif
+	    }
+	}
+#if DEBUG_TS_ENGINE
+      else
+	{
+	  fprintf(stderr, "Couldn't locate user's %s\n", OCHUSHA_INIT_SCM);
+	}
+#endif
+    }
 }
 
 
@@ -847,10 +947,7 @@ open_bulletin_board(OchushaApplication *application,
     OCHU_OBJECT_REF(G_OBJECT(view));
     if (info->threadlist_source_buffer != NULL)
       OCHU_OBJECT_REF(G_OBJECT(info->threadlist_source_buffer));
-#if DEBUG_WIDGET
-    fprintf(stderr, "#1: threadlist_view->ref_count=%d\n",
-	    G_OBJECT(view)->ref_count);
-#endif
+
     commit_job(job);
   }
   THREAD_LIST_UNLOCK;
@@ -1036,10 +1133,7 @@ refresh_threadlist_view(OchushaApplication *application,
     OCHU_OBJECT_REF(G_OBJECT(info->threadlist_source_buffer));
     OCHU_OBJECT_REF(G_OBJECT(tab_label));
     OCHU_OBJECT_REF(G_OBJECT(threadlist_view));
-#if DEBUG_WIDGET
-    fprintf(stderr, "#2: threadlist_view->ref_count=%d\n",
-	    G_OBJECT(threadlist_view)->ref_count);
-#endif
+
     commit_job(job);
 
     OCHU_OBJECT_UNREF(G_OBJECT(old_buffer));
@@ -1110,10 +1204,7 @@ redraw_threadlist_view(OchushaApplication *application,
     OCHU_OBJECT_REF(G_OBJECT(info->threadlist_source_buffer));
     OCHU_OBJECT_REF(G_OBJECT(tab_label));
     OCHU_OBJECT_REF(G_OBJECT(threadlist_view));
-#if DEBUG_WIDGET
-    fprintf(stderr, "#3: threadlist_view->ref_count=%d\n",
-	    G_OBJECT(threadlist_view)->ref_count);
-#endif
+
     commit_job(job);
   }
   THREAD_LIST_UNLOCK;
@@ -1392,12 +1483,6 @@ show_threadlist(WorkerThread *employee, gpointer args)
   OCHU_THREADS_LEAVE();
 
  finish_job:
-#if DEBUG_ASYNC_BUFFER_MOST
-  fprintf(stderr, "show_threadlist: unref OchushaAsyncBuffer(%p)\n",
-	  buffer);
-  fprintf(stderr, "* before unref: buffer->ref_count=%d\n",
-	  G_OBJECT(buffer)->ref_count);
-#endif
 
   OCHU_THREADS_ENTER();
 
@@ -1488,23 +1573,57 @@ update_threadlist_entry_style(OchushaApplication *application,
   if (info->view_flags & BBS_THREAD_FAVORITE)
     visual_parameter.flags |= THREADLIST_ENTRY_FLAG_MARK;
 
-  if (info->view_flags & BBS_THREAD_HIDDEN)
+  if (engine != NULL && list_entry_decorate_func != NULL)
     {
-      visual_parameter.foreground_color = LIST_ENTRY_FOREGROUND_HIDDEN;
-      visual_parameter.background_color = LIST_ENTRY_BACKGROUND_HIDDEN;
+#define TS_BOOL(value)	((value) ? engine->t : engine->f)
+      gboolean filter_result
+	= threadlist_filter_check_thread(&board_info->filter,
+					 application, thread);
+      TSCellHandle *result
+	= ts_engine_evalf0(engine, list_entry_decorate_func,
+			   "%C%C%C%C%C%C",
+			   TS_BOOL(exist_new_responses),
+			   TS_BOOL(board_info->enable_filter),
+			   TS_BOOL(filter_result),
+			   TS_BOOL(info->view_flags & BBS_THREAD_NEW),
+			   TS_BOOL(info->view_flags & BBS_THREAD_FAVORITE),
+			   TS_BOOL(info->view_flags & BBS_THREAD_HIDDEN));
+
+      if (ts_cell_handle_is_pair(result))
+	{
+	  TSCellHandle *fg = ts_cell_handle_get_car(result);
+	  TSCellHandle *bg = ts_cell_handle_get_cdr(result);
+
+	  visual_parameter.foreground_color
+	    = (int)ts_cell_handle_get_long_value(fg);
+	  visual_parameter.background_color
+	    = (int)ts_cell_handle_get_long_value(bg);
+
+	  g_object_unref(G_OBJECT(fg));
+	  g_object_unref(G_OBJECT(bg));
+	}
+      g_object_unref(G_OBJECT(result));
     }
-  else if (info->view_flags & BBS_THREAD_NEW)
+  else
     {
-      visual_parameter.foreground_color = LIST_ENTRY_FOREGROUND_STRONG;
-      visual_parameter.background_color = LIST_ENTRY_BACKGROUND_STRONG;
-    }
-  else if (threadlist_filter_check_thread(&board_info->filter,
-					  application, thread))
-    {
-      if (exist_new_responses)
-	visual_parameter.foreground_color = LIST_ENTRY_FOREGROUND_EMPH;
-      if (!board_info->enable_filter)
-	visual_parameter.background_color = LIST_ENTRY_BACKGROUND_EMPH;
+      if (info->view_flags & BBS_THREAD_HIDDEN)
+	{
+	  visual_parameter.foreground_color = LIST_ENTRY_FOREGROUND_HIDDEN;
+	  visual_parameter.background_color = LIST_ENTRY_BACKGROUND_HIDDEN;
+	}
+      else if (info->view_flags & BBS_THREAD_NEW)
+	{
+	  visual_parameter.foreground_color = LIST_ENTRY_FOREGROUND_STRONG;
+	  visual_parameter.background_color = LIST_ENTRY_BACKGROUND_STRONG;
+	}
+      else if (threadlist_filter_check_thread(&board_info->filter,
+					      application, thread))
+	{
+	  if (exist_new_responses)
+	    visual_parameter.foreground_color = LIST_ENTRY_FOREGROUND_EMPH;
+	  if (!board_info->enable_filter)
+	    visual_parameter.background_color = LIST_ENTRY_BACKGROUND_EMPH;
+	}
     }
 
   threadlist_view_update_thread(view, thread, info->num_shown,
@@ -1771,13 +1890,6 @@ free_response_source_buffer(OchushaBBSThread *thread, gpointer unused)
 {
   BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
 
-#if DEBUG_ASYNC_BUFFER_MOST
-  fprintf(stderr,
-	  "free_response_source_buffer: unref OchushaAsyncBuffer(%p)\n",
-	  info->response_source_buffer);
-  fprintf(stderr, "* before unref: response_source_buffer->ref_count=%d\n",
-	  G_OBJECT(info->response_source_buffer)->ref_count);
-#endif
   if (info->response_source_buffer != NULL)
     {
       OCHU_OBJECT_UNREF(G_OBJECT(info->response_source_buffer));
@@ -1965,9 +2077,12 @@ page_switched_cb(PanedNotebook *paned_notebook,
   info = ensure_bbs_thread_info(thread);
   if (info->next_mark != NULL)
     {
-      BBSThreadView *thread_view
-	= BBS_THREAD_VIEW(gtk_bin_get_child(GTK_BIN(new_page)));
-      bbs_thread_view_scroll_to_mark(thread_view, info->next_mark);
+      GtkWidget *child = gtk_bin_get_child(GTK_BIN(new_page));
+      if (info->recent_view == child)
+	bbs_thread_view_scroll_to_mark(BBS_THREAD_VIEW(child),
+				       info->next_mark);
+      else
+	info->next_mark = NULL;
     }
 
   if (application->show_threadlist_entry_when_thread_selected)
@@ -2018,18 +2133,6 @@ finalize_bulletin_board_view(OchushaBulletinBoard *board, GtkWidget *widget)
   board_view = PANED_NOTEBOOK(widget);
   paned_notebook_foreach_item(board_view, (GFunc)free_response_source_buffer,
 			      NULL);
-#if DEBUG_WIDGET
-  {
-    GtkWidget *scrolled_window = paned_notebook_get_selector(board_view);
-    if (scrolled_window != NULL)
-      {
-	ThreadlistView *threadlist_view
-	  = THREADLIST_VIEW(gtk_bin_get_child(GTK_BIN(scrolled_window)));
-	fprintf(stderr, "#5: finalize: threadlist_view->ref_count=%d\n",
-		G_OBJECT(threadlist_view)->ref_count);
-      }
-  }
-#endif
 }
 
 
