@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: threadlist_filter.h,v 1.1.1.1 2003/05/10 16:34:26 fuyu Exp $
+ * $Id: threadlist_filter.h,v 1.3 2003/12/11 20:04:09 fuyu Exp $
  */
 
 #ifndef _THREADLIST_FILTER_H_
@@ -91,10 +91,17 @@ typedef struct _RuleModelRelation
 #define THREADLIST_FILTER_RULE_SHOW_DAT_DROPPED_THREAD			'D'
 #define THREADLIST_FILTER_RULE_HIDE_MUCH_IGNORED_THREAD			'i'
 #define THREADLIST_FILTER_RULE_HIDE_HIDDEN_THREAD			'h'
+#define THREADLIST_FILTER_RULE_SHOW_GOOD_TITLE_THREAD			'G'
+#define THREADLIST_FILTER_RULE_HIDE_NO_GOOD_TITLE_THREAD		'g'
 
 
+#if 0	/* XXX */
+#define THREADLIST_FILTER_NORMAL_RULES	"FfSsOoNRrDihGg"
+#define MAXIMUM_LENGTH_OF_RULE_STRING	16
+#else
 #define THREADLIST_FILTER_NORMAL_RULES	"FfSsOoNRrDih"
 #define MAXIMUM_LENGTH_OF_RULE_STRING	14
+#endif
 
 
 struct _ThreadlistFilterSetting
@@ -129,6 +136,8 @@ struct _ThreadlistFilterSetting
   RuleModelRelation show_dat_dropped;
   RuleModelRelation hide_much_ignored;
   RuleModelRelation hide_hidden;
+  RuleModelRelation show_good_title;
+  RuleModelRelation hide_no_good_title;
 
   gchar current_rule_string[MAXIMUM_LENGTH_OF_RULE_STRING];
 
